# This code is copyrighted by Stephan Richter in August, 2000.
# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 4, 2001

import string

from OFS.SimpleItem           import SimpleItem
from Globals                  import HTMLFile
import Globals

from DBTable               import DBTable, DBAttribute
from DBObjectRoleSupport   import DBObjectRoleSupport
from DBObjectUndoSupport   import DBObjectUndoSupport
from DBObjectSearchSupport import DBObjectSearchSupport
from ActionTriggerSupport  import ActionTriggerSupport
from RDB2ObjectSupport     import RDB2ObjectSupport
from App.Management        import Navigation

import DBObjectExceptions


class DBObject(Navigation, DBObjectSearchSupport, DBObjectUndoSupport, RDB2ObjectSupport,
               DBObjectRoleSupport, ActionTriggerSupport, SimpleItem):
    '''DBObject class - Base class for PlanDepot objects'''

    # Is the object footprint volatile (not beeing saved in the ZODB)?
    isVolatile           = 0

    isSQLMethodContainer = 0
    hasDBAttributes      = 1

    # This variale specifies which DB field represents the title in the Zope object
    titleDBRepresentation = None
    
    # this variable contains the fingerprint of all SQL Methods that need to be initialized
    sqlMethods = ()

    # The dbTable describes that table and its structure that supports this DBObject
    dbTable = DBTable('DBObject', [],
                      {'containerid'   : DBAttribute('containerid',   'int8', 'int'),
                       'containertype' : DBAttribute('containertype', 'varchar(100)')}
                     )

    # variables that must be defined for Zope
    meta_type = 'DBObject'
    icon = None

    # set the permissions required for executing a certain management method
    __ac_permissions__ = ( (
        ('View management screens', ('manage_main', 'manage_editForm',)),
        ('Edit DBObjects',          ('manage_edit',)),
        ('Delete DBObjects',        ('delete', 'deleteDBAttributes',)),
        ('View DBObjects',          ('getAttributes',)), )
        + DBObjectRoleSupport.__ac_permissions__
        + DBObjectUndoSupport.__ac_permissions__ )

    # The folder will always have an attribute overview
    manage_options = (({'label':'Edit',    'action':'manage_editForm'},)
                      + DBObjectRoleSupport.manage_options 
                      + DBObjectUndoSupport.manage_options)

    manage_main = manage_editForm = HTMLFile('dtml/dbObject_manageEditForm', globals())


    def __init__(self, parent, REQUEST=None, dbRow=None):
        '''__init__(self, parent, REQUEST=None, dbRow=None) --> Initialize the object; return None'''

        # temporary attributes, so that the DB data can be stored without being saved
        self._v_temporaryAttributes = dbRow

        if self.isVolatile:
            self.sqlContainer = self._getSQLMethodContainer(parent)

        # if the REQUEST contains attributes, generate a new database entry using the
        # passed attributes
        if REQUEST:
            localNameSpace = {'self': self, 'REQUEST': REQUEST,
                              'sqlMethodContainer': self._getSQLMethodContainer(parent)}

            # if the addObjectToDB attribute is None, autogenerate the method name
            addMethod = 'sql_add%s' %self.meta_type

            # try to execute the DB insert; if it fails it is assumed that an attribute
            # was missing
            try:
                id = eval('sqlMethodContainer.%s(REQUEST)[0][0]' %addMethod,
                      globals(), localNameSpace) 
            except Exception, error:
                raise DBObjectExceptions.MissingAttributes(self.meta_type, error)

            # Make sure the id is a string
            self.id = str(id)

            # Try to generate a title as well
            if self.titleDBRepresentation and REQUEST.has_key(self.titleDBRepresentation):
                self.title = REQUEST[self.titleDBRepresentation]

        elif dbRow:
            # Make sure the id is a string
            self.id = str(getattr(dbRow, 'id'))

            # if the Zope object gets its title from any DB field, then pass the db field value as
            # the title
            if self.titleDBRepresentation:
                self.title = getattr(dbRow, self.titleDBRepresentation)

        else:
            # if not REQUEST or a dbRow were passed, the initialization is interrupted
            raise DBObjectException.InitializationFailure(self.meta_type)


    def delete(self, REQUEST=None):
        '''delete(self) --> Deletes all its DB and ZODB references; return None'''
        # delete the DB entry before deleting the footprint
        if self.useUndo:
            self.addRevision()
        self.aq_parent.manage_delObjects(ids=[self.id])


    def _getSQLMethodContainer(self, object):
        '''_getSQLMethodContainer(self, object) --> find the SQL Method Container; return object'''

        if self.isVolatile and hasattr(self, 'sqlContainer'):
            return self.sqlContainer
        elif self.isVolatile:
            return object._getSQLMethodContainer(object)
        else:
            # if the object is the SQL method container itself, return it, otherwise find it
            if object.isSQLMethodContainer:
                return object
            else:
                # see whether we have a parent; if not: ouch!
                if object.getParentNode() is not None:
                    return self._getSQLMethodContainer(object.getParentNode())
                else:
                    raise 'SQL Methods Container not found!!! Yikes!'


    def manage_edit(self, REQUEST, forward = 1):
        '''manage_edit(self, REQUEST, forward = 1) --> edit this object using the
           attributes passed in REQUEST. If forward is false, then return nothing else return the
           default view; return None or HTML'''

        self._beforeEdit(self.aq_parent, REQUEST)

        # generate an appropriate local name space
        REQUEST['id']=self.id
             
        localNameSpace = {'self': self, 'REQUEST': REQUEST,
                          'sqlMethodContainer': self._getSQLMethodContainer(self)}

        # if the editObjectInDB attribute is None, autogenerate the method name
        editMethod = 'sql_edit%s' %self.meta_type
            
        # try to execute the DB insert; if it fails it is assumed that an attribute
        # was missing
        try:
            id = eval('sqlMethodContainer.%s(REQUEST)' %editMethod,
                      globals(), localNameSpace) 
        except Exception, error:
            raise DBObjectExceptions.MissingAttributes(self.meta_type, error)

        try: self.title = REQUEST[self.titleDBRepresentation]
        except: pass
        
        if self.useUndo:
            self.addRevision()

        # Update also the temporary attributes. 
	# NOTE: If you write your own edit/update method, you must do the same there!!!
	self.setTemporaryAttributes(self.getAttributes()[0])

        self._afterEdit(self.aq_parent, REQUEST)

        # if forward is false return nothing
        if not forward:
            return None

        if REQUEST is not None:
            return self.manage_editForm(self, REQUEST)        


    def setTemporaryAttributes(self, dbRow):
        '''setTemporaryAttributes(self, dbRow) --> set the temporary attributes in a volatile object
           attributes; return None'''
        self._v_temporaryAttributes = dbRow


    def getTemporaryAttributes(self):
        '''getTemporaryAttributes(self) --> set the temporary attributes in a volatile object
           attributes; return dbRow'''
        if not (hasattr(self, '_v_temporaryAttributes') and self._v_temporaryAttributes):
            self.setTemporaryAttributes(self.getAttributes()[0])
        return self._v_temporaryAttributes

    def getAttributes(self):
        '''getAttributes(self) --> get all properties of this object from the DB; return db
           result object'''

        # create an appropriate local name space
        localNameSpace = {'self': self,
                          'sqlMethodContainer': self._getSQLMethodContainer(self)}

        # if the getObjectFromDB attribute is None, autogenerate the method name
        getMethod = 'qry_get%s' %self.meta_type

        # return the DB result 
        return eval('sqlMethodContainer.%s(id=self.id)' %getMethod,
                    globals(), localNameSpace)


    def deleteDBAttributes(self):
        '''deleteAttributes(self) --> Delete the object in the DB; return None'''

        # create an appropriate local name space
        localNameSpace = {'self': self,
                          'sqlMethodContainer': self._getSQLMethodContainer(self)}

        # if the deleteObjectInDB attribute is None, autogenerate the method name
        deleteMethod = 'sql_delete%s' %self.meta_type

        # return the DB result 
        return eval('sqlMethodContainer.%s(id=self.id)' %deleteMethod,
                    globals(), localNameSpace)


    def getId(self):
        '''getId(self) --> get id of this object; return string'''
        return self.id
        

Globals.default__class_init__(DBObject)
