# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 4, 2001

import string

from AccessControl.Role   import RoleManager
from ActionTriggerSupport import ActionTriggerSupport


class DBObjectRoleSupport(RoleManager, ActionTriggerSupport):
    '''DBObjectRoleSupport class - This class adds some advanced Role support to DBObjects'''

    isRoleSupportRoot   = 0

    __ac_permissions__=(
        ('Add DBObjects',           ('hasManagePermission',)),
        ('Edit DBObjects',          ('hasManagePermission',)),
        ('Delete DBObjects',        ('hasManagePermission',)),
        ('View DBObjects',          ('hasViewPermission',)),
	('Change permissions',      ('addViewRoles', 'addManageRoles', 
	                             'removeViewRoles', 'removeManageRoles')),
        )

    # set the roles that are required to access certain parts of the objects
    manageRoles      = ()
    viewRoles        = ()
    localManageRoles = ()
    localViewRoles   = ()
    superRoles       = ('Manager',)


    def _afterAdd(self, parent=None, request=None):
        '''_afterAdd(self, parent=None, request=None) --> Method executed after the object was added to the ZODB;
           object-specific initializations are done here; return None'''
        self.registerRoles()
        self.afterAdd(parent, request)


    def _makeRoleFillInDictionary(self):
        '''_makeRoleFillInDictionary(self) --> create a dictionary that contains all the ids of all
           objects in the upper tree; return dict'''

        idString = string.lower(self.meta_type)+'id'

        if self.isRoleSupportRoot:
            return {idString: self.id}
        else:
            if hasattr(self, 'aq_parent') and not self.isVolatile:
                dict = self.aq_parent._makeRoleFillInDictionary()
            else:
                dict = {}
            dict[idString] = self.id
            return dict


    def _prepareRoles(self, roles):
        '''_prepareRoles(self, roles) --> prepare the roles for creation; return tuple'''

        ids = self._makeRoleFillInDictionary()
        try:
            parent = self.aq_parent
            ids['parentobjectid'] = parent.id
	    ids['parentobjecttype'] = parent.meta_type
        except:
            ids['parentobjectid'] = ''
	    ids['parentobjecttype'] = ''

            
        try:
            grandparent = self.aq_parent.aq_parent
            ids['grandparentobjectid'] = grandparent.id
            ids['grandparentobjecttype'] = grandparent.meta_type
        except:
            ids['grandparentobjectid'] = ''
            ids['grandparentobjecttype'] = ''

        preparedRoles = []
        
        for role in roles:
            preparedRoles.append(role %ids) 

        return tuple(preparedRoles)


    def getAllRolesInTree(self):
        '''getAllRolesInTree(self) --> get Roles of the object and all its parents;
           return tuple of (manageRoles, viewRoles)'''

        preparedManageRoles = self._prepareRoles(self.manageRoles)
        preparedViewRoles   = self._prepareRoles(self.viewRoles)

        if self.isRoleSupportRoot:
            return (preparedManageRoles, preparedViewRoles)
        else:
            if self.aq_parent is not None:
                roles = self.aq_parent.getAllRolesInTree()
                return (roles[0] + preparedManageRoles, roles[1] + preparedViewRoles)
            else:
                raise 'Yikes! We are at the end of the tree and no isRoleSupportRoot was found!'
    
    
    def addViewRoles( self, roles, local=0 ):
    	'''add local view roles after the fact'''
	if local: 
	    objectRoles = self.localViewRoles
	else:
	    objectRoles = self.viewRoles
	objectRoles = objectRoles + tuple(roles)
	self.registerRoles()
	
    
    def addManageRoles( self, roles, local=0 ):
    	'''add local manage roles after the fact'''
	if local: 
	    objectRoles = self.localManageRoles
	else:
	    objectRoles = self.manageRoles
	objectRoles = objectRoles + tuple(roles)
	self.registerRoles()


    def removeViewRoles( self, roles, local=0 ):
    	'''remove specified local view roles after the fact'''
	if local: 
	    objectRoles = self.localViewRoles
	else:
	    objectRoles = self.viewRoles
	objectRoles = tuple(filter(lambda x,y=roles: not x in y, objectRoles))
	self.registerRoles()
	
    
    def removeManageRoles( self, roles, local=0 ):
    	'''remove specified local manage roles after the fact'''
	if local: 
	    objectRoles = self.localManageRoles
	else:
	    objectRoles = self.manageRoles
	objectRoles = tuple(filter(lambda x,y=roles: not x in y, objectRoles))
	self.registerRoles()


    def registerRoles(self):
        '''registerRoles(self) --> register roles for the DBObjects; return None'''

        manageRoles, viewRoles = self.getAllRolesInTree()
        manageRoles = manageRoles + self.localManageRoles
        viewRoles   = viewRoles   + self.localViewRoles

        for role in manageRoles + viewRoles:
            self._addRole(role)
        
        self.manage_permission('View DBObjects',   self.superRoles + manageRoles + viewRoles, acquire=0)
        
        for action in ['Add', 'Edit', 'Delete']:
            if action+' DBObjects' in map(lambda x: x[0], self.__ac_permissions__):
                self.manage_permission(action+' DBObjects',    self.superRoles + manageRoles, acquire=0)


    def hasViewPermission(self):
        '''hasViewPermission(self) --> execute method to see whether user has view rights; return boolean'''
        return 1


    def hasManagePermission(self):
        '''hasManagePermission(self) --> execute method to see whether user has manage rights; return boolean'''
        return 1

